@@ -6,12 +6,6 @@
 require 'xtemplate'
 
 module XTemplate
-  module XData
-    def to_hash()
-      raise(NotImplementedError, "This method should be implemented for XTemplate")
-    end
-  end
-
   module XPath
     include Util
 
@@ -30,6 +24,38 @@ module XTemplate
       include XPath
       include Util
 
+      def _alt_exist?(name)
+        @alts ||= {}
+        @alts.key?(name)
+      end
+
+      def _alt(name)
+        @alts ||= {}
+        @alts[name] = true
+      end
+
+      def _unalt(name)
+        @alts.delete(name)
+      end
+
+      def alt(val, *args)
+        args.each{|arg|
+          if( _alt_exist?(arg) )
+            val = nil
+          else
+            _alt(arg)
+          end
+        }
+        val
+      end
+
+      def unalt(val, *args)
+        args.each{|arg|
+          _unalt(arg)
+        }
+        val
+      end
+
       def attr(val, *args)
 	case val
 	when Hash
@@ -160,8 +186,7 @@ module XTemplate
 	    newval = val.sort{|x,y|
 	      path.split("/").each{|key|
 		key.strip!
-		if( (x.is_a?(Hash) || (x.is_a?(XData) && (x = x.to_hash))) &&
-		   (y.is_a?(Hash) || (y.is_a?(XData) && (y = y.to_hash))) )
+		if( x.is_a?(Hash) && y.is_a?(Hash) )
 		  x,y = x[key], y[key]
 		end
 	      }
@@ -237,7 +262,7 @@ module XTemplate
 	when Array
 	  newval = val.reject{|v|
 	    case v
-	    when Hash,XData,Array
+	    when Hash, Array
 	      false
 	    else
 	      v.to_s() =~ /\A\s*\z/
@@ -282,10 +307,6 @@ module XTemplate
 	      v = v.dup
 	      v[tag] = idx
 	      newval.push(v)
-	    when XData
-	      v = v.to_hash.dup
-	      v[tag] = idx
-	      newval.push(v)
 	    else
 	      v = {TextNode => v, tag => idx}
 	      newval.push(v)
@@ -310,10 +331,6 @@ module XTemplate
 	      elem.each{|k,v|
 		newval[k] = v
 	      }
-	    when XData
-	      elem.to_hash.each{|k,v|
-		newval[k] = v
-	      }
 	    else
 	      text.concat(elem.to_s)
 	    end
@@ -516,7 +533,7 @@ module XTemplate
 	val.delete(ParentNode)
 	"{" + val.collect{|k,v| "#{k.inspect}=>#{value_inspect(v)}" }.join(", ") + "}"
       when Array
-	"#{val.type}[" + val.collect{|v| value_inspect(v)}.join(", ") + "]"
+	"#{val.class}[" + val.collect{|v| value_inspect(v)}.join(", ") + "]"
       when SanitizedString
 	val.to_s.inspect
       else
@@ -537,17 +554,6 @@ module XTemplate
 	  end
 	}
 	max + 1
-      when XData
-	max = 0
-	val.to_hash.each{|key,val|
-	  if( key == ParentNode )
-	    next
-	  end
-	  if( (x = value_depth(val)) > max )
-	    max = x
-	  end
-	}
-	max + 1
       when Array
 	max = 0
 	val.each{|val|
@@ -583,12 +589,10 @@ module XTemplate
 	    parent.add_child(node)
 	  end
 	}
-      when XData
-	value_to_xml(val.to_hash,parent)
       when Array
 	val = val.collect{|v|
 	  case v
-	  when Hash,XData,Array
+	  when Hash, Array
 	    v
 	  else
 	    {TextNode => v}
@@ -766,9 +770,6 @@ module XTemplate
     end
 
     def normalize(val)
-      if( val.is_a?(XData) )
-	val = val.to_hash()
-      end
       case val
       when Hash
 	if( val[TextNode] && val.size == 1 )
@@ -902,7 +903,17 @@ module XTemplate
 	end
 
 	if( hd.size > 0 )
+          hd,recv,*rargs = hd.split(".")
 	  val = data[hd]
+          if( recv )
+            if( recv =~ /(.+)\((.*)\)/ )
+              recv  = $1
+              rargs = $2.split(',').collect{|c| eval_expr(c, data, plugin) }
+            else
+              rargs = []
+            end
+            val = val.__send__(recv,*rargs)
+          end
 	else
 	  val = data
 	end
@@ -950,7 +961,17 @@ module XTemplate
 
     VALUE_BY_PATH2_2 = %q`
 	if( hd.size > 0 )
+          hd,recv,*rargs = hd.split(".")
 	  val = data[hd]
+          if( recv )
+            if( recv =~ /(.+)\((.*)\)/ )
+              recv  = $1
+              rargs = $2.split(',').collect{|c| eval_expr(c, data, plugin) }
+            else
+              rargs = []
+            end
+            val = val.__send__(recv,*rargs)
+          end
 	else
 	  val = data
 	end
@@ -967,8 +988,6 @@ module XTemplate
 	    case x
 	    when Hash
 	      value_by_path(ids, x, data, rdata, plugin)
-	    when XData
-	      value_by_path(ids, x.to_hash, data, rdata, plugin)
 	    else
 	      if( ids.empty? )
 		x
@@ -1002,9 +1021,6 @@ module XTemplate
     use_default_xpath()
 
     def eval_expr(expr, val, plugin)
-      if( val.is_a?(XData) )
-	val = val.to_hash()
-      end
       case expr
       when "text()"
 	if( val.is_a?(Hash) && val[TextNode] && val.size == 1 )
@@ -1118,9 +1134,6 @@ module XTemplate
 	}
 	return newval
       end
-      if( val.is_a?(XData) )
-	val = val.to_hash()
-      end
       if( act =~ /^([^\(\)]+)\(([^\(\)]*)\)$/ )
 	func = $1.strip
 	args = args_split($2)
